/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "PlayerItem.hpp"

PlayerItem::PlayerItem() : m_mode(MODE_HUMAN), m_width(0), m_height(0)
{
    setChooserId(0);
}

PlayerItem::PlayerItem(float width, float height, const Sprite &background, unsigned int chooserId) :
m_mode(MODE_HUMAN)
{
    setWidth(width);
    setHeight(height);
    setBackground(background);
    
    setChooserId(chooserId);
    
    m_aliasText.setColor( Color(255, 255, 255) );
    m_idText.setColor( Color(255, 255, 255) );
    m_idText.setScale(.8);
}

void PlayerItem::draw()
{
    //Update matrix
    Drawable::pushMatrix();
    Drawable::updateMatrix();
    
    //Draw
    m_background.draw();
    m_animation.draw();
    
    m_aliasText.draw();
    m_idText.draw();
    
    m_modeButton.draw();
    
    Drawable::popMatrix();
}

//Widget methods
bool PlayerItem::mouseClick(const Vector &mouseCoords)
{
    Vector localMouse = Widget::localCoords(mouseCoords);
    
    if (m_modeButton.mouseClick(localMouse))
    {
        m_mode = (m_mode == MODE_AI) ? MODE_HUMAN : MODE_AI;
        
        if (m_mode == MODE_HUMAN) m_modeButton.setText("Human");
        else m_modeButton.setText("AI");
    }
    
    return Widget::contains(localMouse);
}

bool PlayerItem::mouseMove(const Vector &mouseCoords)
{
    return Widget::contains( Widget::localCoords(mouseCoords) );
}

void PlayerItem::update(float frameTime)
{
    //Update animation
    if (m_animation.getCurrentTime() == m_animation.getDuration())
        m_animation.goToTime(0);
    
    m_animation.goForward(frameTime / 1000);
    
    //origin
    Box bodyBox;
    m_animation.getBodyBox(bodyBox);
    m_animation.setOrigin(bodyBox.left, bodyBox.top + bodyBox.height);
}

void PlayerItem::changePlayer(const Animation &animation, const std::string &alias)
{
    m_animation = animation;
    m_alias = alias;
    
    //Scale the animation to half of the chooser's width (nearly) without extending it
    //float scale = (m_width / (1.8 * m_animation.getWidth()) < 1) ? m_width / (1.8 * m_animation.getWidth()) : 1;
    float scale = (m_animation.getWidth() > 0) ? m_width / (1.8 * m_animation.getWidth()) : 1;
    
    if (m_animation.getHeight() * scale > .9 * m_height)
        scale = (m_animation.getHeight() > 0) ? m_height / (1.1 * m_animation.getHeight()) : 1;
    
    m_animation.setScale(scale);
    
    //Center of the view (the origin is at the bottom of the animation)
    m_animation.setYPosition((m_height + m_animation.getHeight() * m_animation.getYScale()) / 2);
    m_animation.setXPosition(.05 * m_width);
    
    //Update alias text
    m_aliasText.setText(m_alias);
    m_aliasText.setPosition(.93 * m_width - m_aliasText.getWidth(), .13 * m_height);
}

void PlayerItem::setChooserId(unsigned int chooserId)
{
    m_chooserId = chooserId;
    
    //Update id text
    std::ostringstream text;
    text << m_chooserId;
    
    m_idText.setText("#" + text.str());
    m_idText.setXPosition(.93 * m_width - m_idText.getWidth() * m_idText.getXScale());
    m_idText.setYPosition(.95 * m_height - m_idText.getHeight() * m_idText.getYScale());
}

unsigned int PlayerItem::getChooserId() const
{
    return m_chooserId;
}

void PlayerItem::setWidth(float width)
{
    if (width > 0)
    {
        m_width = width;
        
        if (m_background.getWidth() > 0)
            m_background.setXScale( m_width / m_background.getWidth() );
    }
}

float PlayerItem::getWidth() const
{
    return m_width;
}

void PlayerItem::setHeight(float height)
{
    if (height > 0)
    {
        m_height = height;
        
        if (m_background.getHeight() > 0)
            m_background.setYScale( m_height / m_background.getHeight() );
    }
}

float PlayerItem::getHeight() const
{
    return m_height;
}

void PlayerItem::setBackground(const Sprite &background)
{
    m_background = background;
    
    if (m_background.getWidth() > 0 && m_background.getHeight() > 0)
    {
        m_background.setPosition(0, 0);
        m_background.setScale( m_width / m_background.getWidth(), m_height / m_background.getHeight() );
    }
}

const Sprite &PlayerItem::getBackground() const
{
    return m_background;
}

void PlayerItem::setTextFont(Font &textFont)
{
    m_aliasText.setFont(textFont);
    m_aliasText.setPosition(.93 * m_width - m_aliasText.getWidth(), .13 * m_height);
    
    m_idText.setFont(textFont);
    m_idText.setXPosition(.93 * m_width - m_idText.getWidth() * m_idText.getXScale());
    m_idText.setYPosition(.95 * m_height - m_idText.getHeight() * m_idText.getYScale());
}

const Font &PlayerItem::getTextFont() const
{
    return m_aliasText.getFont();
}

void PlayerItem::setModeButton(const TextButton &modeButton)
{
    m_modeButton = modeButton;
    
    //Update text
    if (m_mode == MODE_HUMAN) m_modeButton.setText("Human");
    else m_modeButton.setText("AI");
    
    //Update position
    m_modeButton.setPosition(.90 * m_width - m_modeButton.getWidth(), .5 * m_height);
}

const TextButton &PlayerItem::getModeButton() const
{
    return m_modeButton;
}

const std::string &PlayerItem::getAlias() const
{
    return m_alias;
}

unsigned int PlayerItem::getMode() const
{
    return m_mode;
}
