/*Copyright (C) 2010-2012 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Options.hpp"
#include "Engines/GameEngine.hpp"

Options::Options(GameEngine *gameEngine) :
m_gameEngine(gameEngine), m_window(gameEngine->getWindow()),
m_configuration(gameEngine->getConfiguration())
{
    
}

void Options::run()
{
    //Init variables
        //Get view size
    Vector viewSize = Vector(m_window->getViewWidth(), m_window->getViewHeight());
    
    //Background ( + Title)
    TextureManager textureManager;
    FontManager fontManager;
    SoundBufferManager soundBufferManager;
    
    Sprite background( (*textureManager.getTexture("data/gfx/ui/title.png")), Box(0, 0, 1280.f / 2048, 720.f / 1024) );
    background.setScale(viewSize.x / background.getWidth(), viewSize.y / background.getHeight());
    
    Text title("Options", *fontManager.getFont("data/gfx/fonts/default.ttf", 64));
    title.setColor(Color(255, 255, 255));
    
    title.setXPosition((viewSize.x - title.getWidth()) / 2);
    title.setYPosition(30);
    
    //Content
    SoundEngine soundEngine;
    soundEngine.setManager(soundBufferManager);
    soundEngine.setVolume(m_configuration->getMasterVolume() * m_configuration->getSoundVolume() / 100.f);
    
    MainTab mainTab(*fontManager.getFont("data/gfx/fonts/default.ttf", 15), (*m_configuration), textureManager, soundEngine, 900, 400, Color(66, 112, 174));
    mainTab.setPosition((viewSize.x - mainTab.getWidth()) / 2, 135);
    
    KeyBindTab keyTab_p1((*m_configuration), 1, textureManager, (*fontManager.getFont("data/gfx/fonts/default.ttf", 30)),
    (*fontManager.getFont("data/gfx/fonts/default.ttf", 15)), soundEngine);
    keyTab_p1.setPosition(mainTab.getXPosition() + 18, mainTab.getYPosition() + mainTab.getHeight() - keyTab_p1.getHeight() - 18);
    
    KeyBindTab keyTab_p2((*m_configuration), 2, textureManager, (*fontManager.getFont("data/gfx/fonts/default.ttf", 30)),
    (*fontManager.getFont("data/gfx/fonts/default.ttf", 15)), soundEngine);
    keyTab_p2.setPosition(mainTab.getXPosition() + mainTab.getWidth() - keyTab_p2.getWidth() - 18, mainTab.getYPosition() + 18);
    
        //Actions
    Sprite button_default( (*textureManager.getTexture("data/gfx/ui/ui.png")), Box(0, 0, 240.f / 512, 60.f / 512) ),
    button_hover( (*textureManager.getTexture("data/gfx/ui/ui.png")), Box(0, 64.f / 512, 240.f / 512, 60.f / 512) );
    
    //Apply
    TextButton applyButton("Apply", button_default, button_hover);
    applyButton.setSoundEngine(soundEngine);
    applyButton.setFocusSound("data/sound/ui/menuFocus.ogg");
    applyButton.setClickSound("data/sound/ui/menuClick.ogg");
    
    applyButton.setTextFont(*fontManager.getFont("data/gfx/fonts/default.ttf", 24));
    applyButton.setTextColor(Color(255, 255, 255));
    
    applyButton.setXPosition(mainTab.getXPosition() + mainTab.getWidth() - applyButton.getWidth() - 20);
    applyButton.setYPosition(mainTab.getYPosition() + mainTab.getHeight() - applyButton.getHeight() - 20);
    
    //Back
    TextButton backButton("< Back", button_default, button_hover);
    backButton.setSoundEngine(soundEngine);
    backButton.setFocusSound("data/sound/ui/menuFocus.ogg");
    
    backButton.setTextFont(*fontManager.getFont("data/gfx/fonts/default.ttf", 24));
    backButton.setTextColor(Color(255, 255, 255));
    backButton.setPosition(30, 30);
    
    //Playlist handling
    pthread_mutex_t notifierMutex = PTHREAD_MUTEX_INITIALIZER;
    
    PlaylistNotifier notifier;
    notifier.setNotifyTime(2000.f);
    notifier.setFonts(*fontManager.getFont("data/gfx/fonts/default.ttf", 20), *fontManager.getFont("data/gfx/fonts/default.ttf", 12));
    
    notifier.setPosition( viewSize.x - notifier.getWidth() - 20, viewSize.y - notifier.getHeight() - 20 );
    
    PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, &notifier, &notifierMutex);
    
    //Start screen loop
    Vector mouse;
    Color clearColor(255, 255, 255);
    
    SDL_Event event;
    while (1)
    {
        //Manage events
        while (m_window->pollEvent(event))
        {
            switch (event.type)
            {
                case SDL_QUIT: //Application closed
                    m_gameEngine->exit();
                    
                    PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, NULL, NULL);
                    pthread_mutex_destroy(&notifierMutex);
                    return;
                break;
                
                case SDL_VIDEORESIZE: //Window resized
                    m_window->resize(event.resize.w, event.resize.h, false);
                break;
                
                case SDL_KEYDOWN: //Key pressed
                    keyTab_p1.keyDown(event.key.keysym.sym);
                    keyTab_p2.keyDown(event.key.keysym.sym);
                break;
                
                case SDL_MOUSEBUTTONUP: //Mouse clicked
                    mouse.x = event.button.x;
                    mouse.y = event.button.y;
                    
                    if (applyButton.mouseClick(m_window->viewCoords(mouse)))
                    {
                        int windowWidth = m_configuration->getWindowWidth();
                        int windowHeight = m_configuration->getWindowHeight();
                        
                        mainTab.apply(*m_configuration);
                        
                        if (m_configuration->getWindowWidth() != windowWidth || windowHeight != m_configuration->getWindowHeight())
                        {
                            m_window->init(m_configuration->getWindowWidth(), m_configuration->getWindowHeight(), 32, m_configuration->getFullscreen());
                            m_window->initView(m_configuration->getWindowWidth(), m_configuration->getWindowHeight());
                            
                            //Reload
                            m_gameEngine->initOptions();
                            
                            PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, NULL, NULL);
                            pthread_mutex_destroy(&notifierMutex);
                            return;
                        }
                    }
                    else if (backButton.mouseClick(m_window->viewCoords(mouse)))
                    {
                        m_gameEngine->initMainMenu();
                        
                        PlaylistPlayer::playlistSetNotifier(PLAYLIST_MENU, NULL, NULL);
                        pthread_mutex_destroy(&notifierMutex);
                        return;
                    }
                    
                    mainTab.mouseClick(m_window->viewCoords(mouse));
                    keyTab_p1.mouseClick(m_window->viewCoords(mouse));
                    keyTab_p2.mouseClick(m_window->viewCoords(mouse));
                    
                break;
                
                case SDL_MOUSEMOTION: //Mouse moved
                    mouse.x = event.motion.x;
                    mouse.y = event.motion.y;
                    
                    applyButton.mouseMove(m_window->viewCoords(mouse));
                    backButton.mouseMove(m_window->viewCoords(mouse));
                    
                    mainTab.mouseMove(m_window->viewCoords(mouse));
                    keyTab_p1.mouseMove(m_window->viewCoords(mouse));
                    keyTab_p2.mouseMove(m_window->viewCoords(mouse));
                break;
            }
        }
        
        //Display
            //Clear
        m_window->clear(clearColor);
        
            //Draw
        background.draw();
        title.draw();
        
        mainTab.draw();
        keyTab_p1.draw();
        keyTab_p2.draw();
        
        applyButton.draw();
        backButton.draw();
        
            //Playlist notifier
        pthread_mutex_lock(&notifierMutex);
        
        notifier.update();
        notifier.draw();
        
        pthread_mutex_unlock(&notifierMutex);
        
            //Render
        m_window->flush();
    }
}
